@@ -446,10 +446,15 @@ TEST_F(testSquare,noLeafMultivec)
 	rsb_err_t errval = RSB_ERR_NO_ERROR;
 	RSB_REINIT_SINGLE_VALUE_SET(RSB_IO_WANT_LEAF_LEVEL_MULTIVEC ,&wllmv,errval);
 	EXPECT_NO_THROW( on_error( errval ) );
+	rsb_blk_idx_t lc = -1;
+	EXPECT_NO_THROW(rsb_mtx_get_info(mtxAp_, RSB_MIF_LEAVES_COUNT__TO__RSB_BLK_INDEX_T, &lc));
 	errval = rsb_spmm(transA, &alpha, mtxAp_, nrhs, RSB_FLAG_WANT_ROW_MAJOR_ORDER, X.data(), ldB, &beta, Y.data(), ldC);
+	EXPECT_THAT( errval, Ne( RSB_ERR_UNSUPPORTED_TYPE ) );
 	EXPECT_THAT( errval, Eq( RSB_ERR_NO_ERROR ) );
 	errval = rsb_spmv(transA, &alpha, mtxAp_, X.data(), inc, &beta, R.data(), inc);
+	EXPECT_THAT( errval, Ne( RSB_ERR_UNSUPPORTED_TYPE ) );
 	EXPECT_THAT( errval, Eq( RSB_ERR_NO_ERROR ) );
+	if(errval == RSB_ERR_NO_ERROR)
 	for(int i = 0; i < nrA; ++i)
 	{
 		EXPECT_THAT( Y[i*nrhs+1], Eq( R[i]) );
@@ -2268,7 +2273,7 @@ TEST_F(test_upd_vals_binop_complex, error)
 
 
 static int foo;
-class test__rndr_misc: public ::testing::Test {
+class test__rndr_misc: public testRsbAsCxxBase, public ::testing::Test {
 	protected:
 	void * pmp {&pmp};
 	const char * filename {""};
@@ -2305,6 +2310,16 @@ TEST_F(test__rndr_misc, err_flags)
 	EXPECT_THAT( errval, Eq(RSB_ERR_UNIMPLEMENTED_YET) );
 }
 
+#if __gnu_linux__
+TEST_F(test__rndr_misc, write_to_empty_filename)
+{
+	const rsb_marf_t rflags {RSB_MARF_EPS_L};
+	struct rsb_mtx_t * mtxAp_ { rsb_mtx_alloc_from_coo_const(VA.data(), IA.data(), JA.data(), 1+0*nnzA, typecode, 1+0*nrA, 1+0*ncA, brA, bcA, RSB_FLAG_NOFLAGS, NULL) };
+	const rsb_err_t errval = rsb_mtx_rndr("", mtxAp_, pmWidth, pmHeight, rflags);
+	EXPECT_THAT( errval, Eq(RSB_ERR_NO_ERROR) );
+	rsb_mtx_free(mtxAp_);
+}
+#endif
 
 TEST_F(test__rndr_misc, err_flags_others)
 {
@@ -2799,4 +2814,66 @@ TEST_F(test__timers, repeat_big_chunks_mintime_hit_maxdt)
 	EXPECT_THAT( times, Lt( maxtimes ) );
 }
 
+#if defined(LIBRSBPP_H_INCLUDED)
+class RsbPP: public ::testing::Test {
+	protected:
+	RsbLib rsblib { };
+	/* deliberately bad combination, to be checked in inner kernel */
+	const rsb_flags_t flags {RSB_FLAG_SYMMETRIC};
+	const rsb_nnz_idx_t nr {1};
+	const rsb_nnz_idx_t nc {2}; // diagonal-aligned symmetric block is non supported
+	const rsb_coo_idx_t roff {0};
+	const rsb_coo_idx_t coff {0};
+	/* these other flags are ok */
+	const rsb_nnz_idx_t nnz {0};
+	const void* VA {};
+	const void* IA {};
+	const void* IP {};
+	const void* JA {};
+	const rsb_coo_idx_t nrhs {1};
+	const rsb_coo_idx_t ldX {1};
+	const void * rhs {};
+	const rsb_coo_idx_t ldY {1};
+	void * out {};
+	const void * alphap {};
+	const rsb_coo_idx_t incx {1};
+	const rsb_coo_idx_t incy {1};
+	const rsb_trans_t transA {RSB_TRANSPOSITION_N };
+	const rsb_flags_t order {RSB_FLAG_WANT_ROW_MAJOR_ORDER};
+};
+
+TEST_F(RsbPP, all_types_ok)
+{
+	for ( rsb_type_t typecode : RSB_MATRIX_TYPE_CODES_ARRAY )
+	{
+		const rsb_err_t errval = rsbpp_coo_spmm(typecode, flags, nnz, nr, nc, VA, IA, JA, nrhs, ldX, rhs, ldY, out, alphap, incx, incy, transA, roff, coff, order); EXPECT_THAT( errval, Ne( RSB_ERR_UNSUPPORTED_TYPE ) );
+		EXPECT_THAT( errval, Ne( RSB_ERR_NO_ERROR ) );
+	}
+
+	for ( rsb_type_t typecode : RSB_MATRIX_TYPE_CODES_ARRAY )
+	{
+		const rsb_err_t errval = rsbpp_csr_spmv(typecode, flags, nnz, nr, nc, VA, IP, JA, rhs, out, alphap, incx, incy, transA, roff, coff);
+		EXPECT_THAT( errval, Ne( RSB_ERR_UNSUPPORTED_TYPE ) );
+		EXPECT_THAT( errval, Ne( RSB_ERR_NO_ERROR ) );
+	}
+}
+
+TEST_F(RsbPP, bad_type_not_ok)
+{
+	const rsb_type_t typecode { RSB_NUMERICAL_TYPE_INVALID_TYPE };
+	{
+		const rsb_err_t errval = rsbpp_coo_spmm(typecode, flags, nnz, nr, nc, VA, IA, JA, nrhs, ldX, rhs, ldY, out, alphap, incx, incy, transA, roff, coff, order);
+		EXPECT_THAT( errval, Eq( RSB_ERR_UNSUPPORTED_TYPE ) );
+		EXPECT_THAT( errval, Ne( RSB_ERR_NO_ERROR ) );
+	}
+
+	{
+		const rsb_err_t errval = rsbpp_csr_spmv(typecode, flags, nnz, nr, nc, VA, IP, JA, rhs, out, alphap, incx, incy, transA, roff, coff);
+		EXPECT_THAT( errval, Eq( RSB_ERR_UNSUPPORTED_TYPE ) );
+		EXPECT_THAT( errval, Ne( RSB_ERR_NO_ERROR ) );
+	}
+}
+
+#endif
+
 #endif /* RSB_WANT_GTEST */
